@@ -8,8 +8,12 @@ from jsonschema.validators import _LATEST_VERSION
 class TestBestMatch(TestCase):
     def best_match_of(self, instance, schema):
         errors = list(_LATEST_VERSION(schema).iter_errors(instance))
+        msg =  f"No errors found for {instance} under {schema!r}!"
+        self.assertTrue(errors, msg=msg)
+
         best = exceptions.best_match(iter(errors))
         reversed_best = exceptions.best_match(reversed(errors))
+
         self.assertEqual(
             best._contents(),
             reversed_best._contents(),
@@ -96,6 +100,35 @@ class TestBestMatch(TestCase):
         best = self.best_match_of(instance=[], schema=schema)
         self.assertEqual(best.validator, "type")
 
+    def test_anyOf_traversal_for_single_sibling_errors(self):
+        """
+        We *do* traverse anyOf with a single subschema that fails multiple
+        times (e.g. on multiple items).
+        """
+
+        schema = {
+            "anyOf": [
+                {"items": {"const": 37}},
+            ],
+        }
+        best = self.best_match_of(instance=[12, 12], schema=schema)
+        self.assertEqual(best.validator, "const")
+
+    def test_anyOf_traversal_for_non_type_matching_sibling_errors(self):
+        """
+        We *do* traverse anyOf with multiple subschemas when one does not type
+        match.
+        """
+
+        schema = {
+            "anyOf": [
+                {"type": "object"},
+                {"items": {"const": 37}},
+            ],
+        }
+        best = self.best_match_of(instance=[12, 12], schema=schema)
+        self.assertEqual(best.validator, "const")
+
     def test_if_the_most_relevant_error_is_oneOf_it_is_traversed(self):
         """
         If the most relevant error is an oneOf, then we traverse its context
@@ -149,6 +182,35 @@ class TestBestMatch(TestCase):
         best = self.best_match_of(instance=[], schema=schema)
         self.assertEqual(best.validator, "type")
 
+    def test_oneOf_traversal_for_single_sibling_errors(self):
+        """
+        We *do* traverse oneOf with a single subschema that fails multiple
+        times (e.g. on multiple items).
+        """
+
+        schema = {
+            "oneOf": [
+                {"items": {"const": 37}},
+            ],
+        }
+        best = self.best_match_of(instance=[12, 12], schema=schema)
+        self.assertEqual(best.validator, "const")
+
+    def test_oneOf_traversal_for_non_type_matching_sibling_errors(self):
+        """
+        We *do* traverse oneOf with multiple subschemas when one does not type
+        match.
+        """
+
+        schema = {
+            "oneOf": [
+                {"type": "object"},
+                {"items": {"const": 37}},
+            ],
+        }
+        best = self.best_match_of(instance=[12, 12], schema=schema)
+        self.assertEqual(best.validator, "const")
+
     def test_if_the_most_relevant_error_is_allOf_it_is_traversed(self):
         """
         Now, if the error is allOf, we traverse but select the *most* relevant
@@ -396,7 +458,33 @@ class TestErrorTree(TestCase):
         tree = exceptions.ErrorTree([error])
         self.assertIsInstance(tree["foo"], exceptions.ErrorTree)
 
-    def test_repr(self):
+    def test_iter(self):
+        e1, e2 = (
+            exceptions.ValidationError(
+                "1",
+                validator="foo",
+                path=["bar", "bar2"],
+                instance="i1"),
+            exceptions.ValidationError(
+                "2",
+                validator="quux",
+                path=["foobar", 2],
+                instance="i2"),
+        )
+        tree = exceptions.ErrorTree([e1, e2])
+        self.assertEqual(set(tree), {"bar", "foobar"})
+
+    def test_repr_single(self):
+        error = exceptions.ValidationError(
+            "1",
+            validator="foo",
+            path=["bar", "bar2"],
+            instance="i1",
+        )
+        tree = exceptions.ErrorTree([error])
+        self.assertEqual(repr(tree), "<ErrorTree (1 total error)>")
+
+    def test_repr_multiple(self):
         e1, e2 = (
             exceptions.ValidationError(
                 "1",
@@ -412,6 +500,10 @@ class TestErrorTree(TestCase):
         tree = exceptions.ErrorTree([e1, e2])
         self.assertEqual(repr(tree), "<ErrorTree (2 total errors)>")
 
+    def test_repr_empty(self):
+        tree = exceptions.ErrorTree([])
+        self.assertEqual(repr(tree), "<ErrorTree (0 total errors)>")
+
 
 class TestErrorInitReprStr(TestCase):
     def make_error(self, **kwargs):
@@ -556,6 +648,29 @@ class TestErrorInitReprStr(TestCase):
             validator="maxLength",
         )
 
+    def test_does_not_reorder_dicts(self):
+        self.assertShows(
+            """
+            Failed validating 'type' in schema:
+                {'do': 3, 'not': 7, 'sort': 37, 'me': 73}
+
+            On instance:
+                {'here': 73, 'too': 37, 'no': 7, 'sorting': 3}
+            """,
+            schema={
+                "do": 3,
+                "not": 7,
+                "sort": 37,
+                "me": 73,
+            },
+            instance={
+                "here": 73,
+                "too": 37,
+                "no": 7,
+                "sorting": 3,
+            },
+        )
+
     def test_str_works_with_instances_having_overriden_eq_operator(self):
         """
         Check for #164 which rendered exceptions unusable when a
@@ -583,5 +698,5 @@ class TestErrorInitReprStr(TestCase):
 
 class TestHashable(TestCase):
     def test_hashable(self):
-        set([exceptions.ValidationError("")])
-        set([exceptions.SchemaError("")])
+        {exceptions.ValidationError("")}
+        {exceptions.SchemaError("")}
